import datetime
import logging
import typing

import dateutil.parser
import pydantic

_logger = logging.getLogger(__name__)

now = datetime.datetime.now()
local_now = now.astimezone()
local_tz = local_now.tzinfo


def date_str_to_datetime(date: str) -> datetime.datetime:
    if not date:
        return None

    dt = dateutil.parser.parse(date).replace(tzinfo=local_tz)
    return dt


class Invoice(pydantic.BaseModel):
    number: int
    submitted_on: typing.Optional[datetime.datetime] = None
    paid_on: typing.Optional[datetime.datetime] = None

    # validators
    # https://docs.pydantic.dev/usage/validators/#reuse-validators
    _normalize_submitted_on = pydantic.validator(
        "submitted_on", pre=True, allow_reuse=True
    )(date_str_to_datetime)

    _normalize_paid_on = pydantic.validator("paid_on", pre=True, allow_reuse=True)(
        date_str_to_datetime
    )


class InvoiceList(pydantic.BaseModel):
    __root__: typing.List[Invoice]


class Task(pydantic.BaseModel):
    task: str
    minutia: typing.Optional[str] = ""
    task_time: str


class TaskList(pydantic.BaseModel):
    __root__: typing.List[Task]


class DailyEntry(pydantic.BaseModel):
    tasks: TaskList
    date: datetime.datetime
    invoice: int
    total_time_sec: typing.Optional[int] = None

    _normalize_date = pydantic.validator("date", pre=True, allow_reuse=True)(
        date_str_to_datetime
    )

    @pydantic.validator("total_time_sec")
    def prevent_none(cls, v):
        assert v is not None, "total_time_sec may not be None"
        return v


class Timesheet(pydantic.BaseModel):
    invoices: InvoiceList
    days: typing.List[DailyEntry]



import csv
import hashlib
import logging
import pathlib
import tempfile

import gspread
import gspread.exceptions
import oauth2client.service_account

from . import config as configmod

_logger = logging.getLogger(__name__)

workbook_name = configmod.config["FOUNDATIONLIVE_GOOGLESHEETS_WORKBOOK_NAME"]
credentials_path = pathlib.Path(
    pathlib.Path(configmod.config["FOUNDATIONLIVE_GOOGLESHEETS_AUTH_JSON_FILE_PATH"])
)

data_dir = credentials_path.parent
last_run_csv_path = data_dir / "cached_last_run.csv"


def get_file_checksum(path: pathlib.Path) -> str:
    if not path.exists():
        return ""

    checksum = None
    with open(path, "rb") as f:
        bytes = f.read()  # read entire file as bytes
        checksum = hashlib.sha256(bytes).hexdigest()

    return checksum


def no_change(data: str) -> bool:
    if not last_run_csv_path.exists():
        return False

    last_hash = get_file_checksum(last_run_csv_path)

    tmpfile = tempfile.NamedTemporaryFile(delete=False)
    contents = data

    path = pathlib.Path(tmpfile.name).resolve()

    data = contents.encode(encoding="UTF-8")
    tmpfile.write(data)
    tmpfile.close()

    new_hash = get_file_checksum(path)

    _logger.debug(f"{new_hash=}")
    _logger.debug(f"{last_hash=}")

    if last_hash == new_hash:
        return True

    return False


def main(csv_file):
    scopes = [
        "https://www.googleapis.com/auth/spreadsheets",
        "https://www.googleapis.com/auth/drive",
    ]

    creds = (
        oauth2client.service_account.ServiceAccountCredentials.from_json_keyfile_name(
            credentials_path, scopes
        )
    )
    client = gspread.authorize(creds)
    workbook = client.open(workbook_name)
    sheet_title = "sheet1"

    if no_change(csv_file):
        msg = "skipping update googlge docs because " "last run is same as current run "
        _logger.debug(msg)
        return

    workbook.values_update(
        sheet_title,
        params={"valueInputOption": "USER_ENTERED"},
        body={"values": list(csv.reader(csv_file.splitlines()))},
    )

    last_run_csv_path.write_text(csv_file)


if __name__ == "__main__":
    main()
